# Logical data transfer between trusted devices 
IP networking is one of the most reliable infrastructures that human beings have built. With the continuous development of software and hardware, more and more devices are connected to the Internet through the IP protocol. One of the common needs of current applications are: to communicate with a specific device from network.

In the IP network, the identification of the device is its IP address, and the application has to define the identity authentication logic to associate the IP address with the trusted device; devices may not communicate with each other directly, for IPV4 address mapping(IPV6 is not yet popular).

For example, user using a mobile phone to send a "start cleaning" command to the in house sweeping robot; based on IP network, developers write the following code:
```
phone side:
1. establish a connection to the control server;
2. send a login command with user name and password, then wait for the control server confirming the identity;
3. login success, sends "start cleaning" command on connection to control server;
```
```
robot side:
1. establish a connection to the control server;
2. send a login command with product serious number, then wait for the control server confirming the identity;
3. login success, waiting for instructions from control server;
4. receive the "start cleaning" command forwarded by the control server from phone side;
```
On BDT protocol, we design the following features to transfer data between trusted devices,  
+ build networks based on device identities rather than IP addresses;
+ identity of another device can be verified decentered on any device;
+ data can be transferred between any devices directly, with encryption; 
the example above, based on BDT, implemention looks like:
```
phone side:
1. establish an encrypted connection to the robot, through the device identification of the robot;
2. sends "start cleaning" command on connection;
```

```
robot side:
1. accepts a incomming connection;
2. verify incoming device identification of connection, verify permission of device;
3. if permission accepted, receive "start cleaning" command on connection;
```

## Device object
BDT use named object system to identify a device, named as `device object`; a device object looks like the following json structure:
```json
{
    "public_key":"xxxxxx",
    "area" : "china",
    "device_type" : "android",
    "device_id" : "A23B7A3", 
}
``` 
the most important field is `public_key`, which points to a public parts of a key pair generated by one kind of asymmetric encryption algorithm; only device's owner keeps the prive parts.


## Tunnel
To build a logic trusted and encrypted path between 2 devices(call them D1 and D2), on which packet passes:
```
1. D1 generate a temporary, symmetric key: K, send a handshake packet to D2 through a link layer protocol;the handshake packet contains K, encrypted with the public key of D2's device object; handshake packet also contains a signature with the private key of D1;
2. D2 uses its private key to decrypt the handshake packet, verifies handshake packet with D1's public key, records K as key for packets from D2.
3. D1 and D2 sends following packet ecrypted with K to each other.
```
We call this logic path between devices a `tunnel`; 

Using Multi-subnet registration/publishing mechanism, `tunnel` can be built on multi different link layer protocols. IP protocol degrades to link layer, this source code implement BDT with IP protocol, supports TCP/UDP, IPv4/IPv6 at the same time. 

There's built-in support for proxy links through a third relay node. Application can select the appropriate IP link to optimize the tunnel's link through a certain strategy.

Obviously, we also solve problems such as NAT hole-punching on traditional P2P area: `Super Node` which has static public IP address are involved.

## Application layer semantic objects
Application programming based on IP protocol rarely sends and receives IP datagrams directly, but uses the transport layer protocol TCP/UDP. Applications based on BDT protocol also rarely use `tunnel` to send and receive packages directly; BDT has built-in implementation of application layer semantic objects based on `tunnel`.

### Stream: 
Full-duplex, reliable data transfer semantics, as TCP on IP. What's more:
+ embed lightweight Q&A in handshake packets;
+ customed congestion control algorithms;

### Datagram:
connectionless unreliable datagram sending and receiving semantics, as UDP on IP.what's more:
+ send datagrams exceed link layer MTU;
+ broadcast over global network;


# Named data transfer
If device knows the digital digest of the data to be received, it can verify the received data through the digest; then there's no necessary to receive data from trusted sources; no encryption, no verification of source identification. Most of the current Internet traffic is such data, e.g. a moive, we call such data `named data`.

To achieve efficient transmission of named data in the network. NDN(Named Data Networking) Theory defines the behavior of terminals and relay nodes in the network, send/cache/discard/forward two types of basic packets: Interest and Data.
A typical scenario is: two devices D1 and D2 on the same router R play a popular movie at the same time. Intuitively, D1 and D2 request the same data D from the media server S, traffic from S to R is 2 * sizeof(D).

On NDN, behavior on router R may be:
```
1. forward D1's interest to S;
2. cache data D sent from S to D1;
3. discard D2's interest to S;
4. send D from R to D2;
```
Then traffic from S to R becomes sizeof(D), overall traffic of the network is optimized.

BDT also implement NDN to transfer `named data`:
+ `Tunnel` aslo supports plaintext packet;
+ `Chunk`: defines a named data;
+ `Channel`: based on plaintext `tunnel`, provides transmission of `chunks` between devices. Compare to `stream`, `channel` has different encoding modes, sending window control, congestion control algorithms.
+ `Task`: requesting one/multi chunk from network semantics;



