# CYFS 协议简介

协议设计是对未来网络工作方式的预言。CYFS协议对未来互联网的数据定义和传输做了全新的设计，让互联网在更好的确保去中心网状结构的前提下，从传输网进化成了价值网和语义网。从实现的角度来看，CYFS协议族是对今天互联网3大基础协议的升级。

TODO：附图

## 从面向地址到面向可信设备
众所周知，IP协议是面向地址的协议。任何一个接入Internet的设备都会获得一个全网唯一的IP地址，知道给地址的其它设备就可与之开始通信。通信的实现依赖遍布全球，网状链接的路由器。已经建成的IP网络是人类构建的最可靠的基础设施之一，该网络对局部的设备损坏容忍力极强。但这个基础设施，越来越无法满足常规开发的需求，随着可连网设备的数量和种类越来越多，大量网络应用通常要使用的逻辑是：在两个可信设备之间进行通信。以智能家居场景为例，用手机向扫地机器人发送一条开始清扫命令，需要编写下面代码

```
手机端:
建立到控制服务器的连接(控制服务器IP地址已知);
发送登录指令，等待控制服务器确认身份;
登录成功，该链接为可信链接;
通过该链接向扫地机器人发送“开始清扫”指令;
```

```
扫地机器人端:
建立到控制服务器的连接（控制服务器IP地址已知);
发送登录指令，等等控制服务器确认身份;
登录成功，等待指令
收到控制服务器转发的“开始清扫”指令;
```
上述例子，手机需要通过一个认证控制服务器来转发指令到扫地机器人。有如下问题：
1.控制服务器（云）需要和大量的智能家居设备保持连接，开发和运维门槛高。
2.设备工作依赖厂商服务器。当控制服务器停机,或无法联通时，所有与之相连的智能家居都无法正常工作。
3.当开发团队的安全架构设计不当，黑客攻击控制服务器后，有机会遥控所有用户的智能家居，造成严重的恶行事故。

使用面向可信设备BDT协议进行改造后，应用开发只需编写如下代码即可
```
手机端：
读取待控制扫地机器人的设备证书;
通过扫地机器人的设备证书，建立到扫地机器人的加密连接;
连接建立成功，发送“开始清扫指令”;
```

```
扫地机器人端:
收到连接建立请求，读取请求端设备证书（手机的设备证书);
确认手机设备证书的拥有者有权限控制机器人，加密连接建立成功;
等待控制指令;
```
BDT协议设计上直接支持可信设备通信后，降低了应用开发的难度。并让典型的IoT系统不再依赖中心化的云基础设施，提高了系统的可靠性和安全性，也能实现系统在无互联网环境的私有部署。

## 可信信道的建立过程
CYFS设计了源自区块链的命名对象(Named Object)机制，BDT协议在此基础上实现了去中心的可信设备的寻址、认证和加密通信。我们先对命名对象机制做一点简单的介绍：

### Named Object 机制
首先，命名对象是结构化的信息。得益于前人的工作，json这样的数据定义方式已经被广泛使用。我们可以用json来定义一个类型为“People”的命名对象：

```json
{
    "name" : "liuzhicong",
    "email" : "liuzhicong@buckyos.com",
    "gender" : "male", 
    "nickname" : "LZC"
}
```

在CYFS中，和这个json语义一致的命名对象称作普通命名对象，这类对象是`不可变对象`。CYFS要求语义上相同的命名对象编码后的结果相同。命名对象的统一编码标准是CYFS命名对象协议的一部分，该标准的设计并不复杂，本文不予阐述。对编码结果进行Hash计算得到该命名对象的`ObjectId`。

```
ObjectId = Hash(Encode(NamedObject))
```

接下来，我们在这个“People”对象中，增加一个重要的字段public_key(使用base58编码)。创建这个People对象的自然人会妥善的保存其对应的私钥。

```json
{
    "public_key":"xxxxxx",
    "name" : "liuzhicong",
    "email" : "liuzhicong@buckyos.com",
    "gender" : "male", 
    "nickname" : "LZC"
}
```

这种包含有公钥的命名对象被称作`有权对象`。能通过签名的方法进行`授权`。在这个设计下，我们能命名对象中加入可变部分。比如，我们希望People的email和nickname是可以修改的，定义如下

```json
{
    "public_key":"xxxxxx",
    "name" : "liuzhicong",
    "gender" : "male",
    
    "mut_body" : {
        "email" : "liuzhicong@buckyos.com",
        "nickname" : "LZC"
    },
    "mut_body_sign" : "xxxxxx"
}
```
对这个有权对象进行ObjectId计算时，不会包含mut_body部分和mut_body_sign部分。

安全可靠的保护私钥需要消耗成本和精力，我们不可能让每一个命名对象都是`有权对象`。`有主对象`才是广泛使用的可`确权`对象。下面是一个代表`文章`的有权对象TopicObject的设计
```json
{
    "owner_id":"xxxxx",
    "title":"CYFS与2021年10月24日公开",
    "publish_time":"2021-10-24 20:48:00",

    "mut_body" : {
        "content" : "文章内容详情"
    },
    "desc_sign":"xxxx",
    "mut_body_sign":"xxxxx"
}
```
其中owner字段的内容是类型为People的ObjectId。使用该TopicObject时，应先得到其Owner Object。通过Owner ObjectId可以对Owner Object的完整性进行校验.随后再用Owner Object中的public key来检验TopicObjet的签名，确定这篇文章是由确定的作者发布的。

基于上述机制，我们可以定义BDT中用到的DeviceObject,DeviceObject是有一个`有主有权对象`。
```json
{
    "owner_id" : "xxxx",
    "publick_key" : "xxxx",
    "area" : "china",
    "device_type" : "x86_pc",
    "device_id" : "A23B7A3", 

    "mut_body" : {
        "nick_name" : "lzc's PC",
        "address" : [
            "v4@192.168.100.201",
            "v4@220.123.11.45",
            "v6@"
        ]
    },
    "desc_sign" : "xxxxx",
    "mut_body_sign" : "xxxx",
}
```
DevceObject包含了网络中设备的关键信息，包括设备的所有者，设备的类型和序列号，在mut_body中，还在mut_body中了包含设备设定的通信地址，地址信息包含了链路协议类型和面向该链路协议的网络地址信息。DeviceObject取代了现在的CA证书。

TODO：由Named Object构成的对象网络例子


### BDT的可信通信
 BDT的底层通信接口定义简化如下:
```rust
fn open_tunnel(local_device_object:DeviceObject, 
               local_device_private_key:PrivateKey,
               remote_device_object:DeviceObject) -> Result<Tunnel>;
```
在与目标设备建立可信信道（Tunnel) 前， 需要准备好本机DevceObject，对应的PrivateKey，以及目标设备的DeviceObject。建立信道的流程如下:
```
1.client根据server的device object信息得到server的可靠可通信地址，发送带交换密钥的握手包给server。握手包用server的public key加密,并使用client的private key签名。
2.servrer用自己的private key解密握手包，记录tunnel的可用密钥。随后可以使用加密通信回复ACK包。
3.加密包使用通用结构。包头为密钥与时间偏移的Hash，包体为加密后的原始内容。
```
BDT的可信信道具有双方身份可信、可信设计不依赖CA证书、加密通信握手快，加密传输包无可见结构等优点。

### 通过ObjectId得到NamedObject

```
let obj = resolve(obj_id);
publish_obj(obj,private_key);
```

按命名对象的机制，有主对象和有权对象，都可以有一个可以可信更新的mut_body，如何得到指定NamedObject的最新版本是整个系统能够高效工作的基础。得益于命名对象机制的命名对象设计，只需要在 mut_body 中加入递增的版本信息，那么不管是从什么渠道得到的NamedObject, 都可以判断谁是`最新`的。CYFS提供了3种设施来实现目的。

首推使用CYFS生态里的MetaChain(元链)来解决这个问题。把MetaChain是用区块链技术改造的新DNS，专门为解决命名对象的解析和更新服务。

其次使用D-DHT，这在一个中型规模，延迟较小的物理网络中会是一个较好的选择。

最后是应用根据自己的场景架设专门的缓存服务，可以根据应用场景在成本和效率之间取得一个平衡。


## BDT替代TCP/IP的实施路径

第一阶段、应用处于降低开发难度的原因，基于BDT协议进行开发。应用增多后，当前IP网络上会存在更多包含BDT协议头的数据包
第二阶段、链路两端的路由器进行升级，识别BDT协议中的命名数据，可以通过拦截相同命名数据的方式降低链路的负载。在不升级物理链路的情况下，获得更大的数据传输能力，进而得到更多的经济效应。这会逐步吸引今天IP网络上所有的重要路由器进行升级。
第三阶段、当IP网络上有足够多的路由器同时支持IP协议和BDT协议后，整个生态会开始向BDT倾斜，建立基于可信设备ID的更高效的数据路由体系，最终完全淘汰IP网络。


## 分离逻辑数据和命名数据
互联网上的数据可以分为两大类：
逻辑数据：通过TCP/IP传输的所有数据都可以看作是逻辑数据，网络只关心报文的source和destion,而不关注报文的Payload. 今天的互联网为了通信安全，这些Payload也大多是处于加密不可读的状态。
命名数据：在传输前，接收端就明确知道数据Hash的数据可以曾作命名数据。互联网上的流量大头都是命名数据。举个例子，一部热门的电影，全网有10万人在同时观看，观看完成后都得到了一样的视频数据。这些视频数据就是命名数据。由于数据接收方事先持有命名数据的Hash，命名数据不惧怕中间人篡改数据。命名数据大多是公开内容，所以命名数据也不需要加密。命名数据的传输可以是来源无关的：只要最好Hash能对上，不在乎
路由器识别BDT命名数据后

BDT协议栈提供了新的编程接口，让应用层可以区分应用内的逻辑数据和命名数据。接口设计如下：
```
信道建立:
fn open_tunnel() ->Tunnel;
逻辑数据:
fn open_stream() -> Stream;
fn send_datagram(tunnel,datagram);
名字数据:
fn interest(chunk_id,Opiton<tunnel>);
```
由于命名数据在传输时默认不加密，传输链路上途径的路由节点可以识别命名数据，并优化链路的性能。典型流程如下图：
TODO：

传输入口的路由器的简单优化策略为拦截已传输的相同命名数据，这样在链路上传输的数据从{D1,D1,D2,D2,D3} 变成了 {D1,D2,D3},出口路由器可还原出{D1,D1,D2,D2,D3}，优化后链路的带宽使用下降了40%。应用这个优化需要升级的路由器能识别BDT协议，还需要有配置一定容量的高速缓存。一次性硬件投入的成本不太大，运营商会基于经济动力去升级路由器。当组成互联网的大部分路由器都能识别并支持BDT协议后，BDT协议就有机会真正替代IP协议。

## 命名数据传输流程 （对NDN理论的改良）
一个典型的命名数据传输流程如下图所示：

1.命名数据的需求者发送interest 命令，该命令可以没有明确的目标
2.链路上的任何路由节点除了路由interest命令外，还可以拦截、转发该命令
3.拥有数据的节点（包括链路上的节点），向数据需求者发送piece.一个命名数据可以编码成多个piece。最简单的分片方式，数据需求方收到下标连续的piece后可以得到命名数据并进行验证。     

BDT协议还支持命名数据在传输时使用网络编码，比如使用喷泉码，让数据需求方收到足够不同下标的piece就可以还原出命名数据。这能更好的利用命名数据的需求者不关注数据来源的特性，在同等物理网络条件下，更快更好的完成数据传输任务。以实时视频会议为例（RTC），可以看到BDT传输组消耗更小的发布者带宽，并拥有更低的平均接收延迟。

TODO:图片

*实验数据：网络平均延迟100ms,在1000人的超大群众，发言节点最高带宽占用为码率的3倍，参与者者的平均延迟为173ms,最高延迟为427ms。

BDT的命名数据设计，不但能让网络应用开发更简单，效果更好，成本更低。

## 其它现代特性
作为一个现代设计的网络协议，BDT还支持一些主流的现代特性，包括
1. 支持DHT，对Mesh结构更友好。BDT的DeviceId设计也让DHT距离与真实距离更接近，形成的网络也更稳固。
2. 通过无协议特征提供更强的安全性，包括全随机协议实现无协议特征，和多跳网络实现无流量特征。
3. 支持广域网广播。通过可验证明文命令以及基于PoW的去中心可信身份，BDT支持一个高信用节点在广域网上进行广播。这能在互联网上更好的保护高价值节点的隐私。


## 基于IP网络的路由
今天，BDT运行在IP网络之上。利用命名对象的可信Body，可以非常简单的实现路由。DeviceObject在其MutBody中，包含一组该设备的可用IP地址，再将这个更新的DeviceObject扩散出去，就可以被链接。对绑定的IP地址经常发生变化的设备（动态设备），我们还专门提供了SuperNode机制来实现可链接性。SuperNode一定使用固定IP地址，动态
设备的DeviceObject的MutBody中，设置自己使用的SuperNode的DeviceId,设备上线会给SuperNode发送上线通知，SuperNode会在通知时得知Device当前的动态IP，其它设备来连接该动态设备时，先与SN通信，得到该设备当前的动态IP后，再与设备通信。（如下图）

TODO：图片

按BDT的设计，IP协议已经从传输层协议下降为链路层协议，使用BDT的应用可以平滑的使用IPv4和IPv6协议。从实验室的反馈来看，使用IPv6的设备的可链接性更好，但带宽质量稍差。BDT的更大范围的应用，可以帮助IPv6、5G网络的的普及高效利用。

## 未来的组网和路由

BDT使用“多子网注册/发布” 机制，允许Internet上存在更多的，不同类型的链路层协议。BDT现在就是基于该机制实现了对IPv4和IPv6的同时支持。这个机制的基本原理

在今天的两个典型新链路层场景里，BDT使用新的机制能比IP网工作的更好（限于篇幅就不展开讲流程了）
无线Mesh：(下图TODO)
量子通信：(下图TODO)


我们预测BDT会和IP协议共存一段不短的时间，所以BDT今天只关注通用机制的设计，未来的最优策略将在正确时间点由那时的人们来确定。

## BDT是cyfs协议族的一部分

TODO：

## 实施情况
基于IP网络的BDT协议已经进入中型规模的线上验证阶段，整个协议栈为全自研，支持目前全部的主流平台。






