# 设计目的
BDT(Bucky Data Trans) 协议旨在为未来互联网提供一种全新的基础网络传输协议。相比TCP/IP协议，BDT提供了更丰富的基础语义。    
BDT协议目前基于TCP/IP协议构建，但也允许未来工作在其它的基础链路协议之上。 下面是BDT协议要解决的核心问题   

## 提供基于设备ID的点到点传输能力
1. 基于设备绑定的网络节点逻辑标识(DeviceId)来构建网络，而不是基于数据路由网络地址(IP地址)。
2. DeviceId的生成是去中心化的。
3. 设备到设备的数据传输是加密、可信的，且不依赖基于根证书体系的中心化证书系统。
4. DeviceId的PoW设计，让BDT网络中的节点拥有了基础的信用等级，也让提供服务的节点拥有了防御DDoS攻击的基础能力，
5. 可控两跳访问的设计，能尽量抹平系统中的流量特征，让提供服务的节点拥有了一定的物理隐秘能力 
6. 协议有现代的Noise能力，从中间人的角度来看是没有任何特征的。
6. 目前基于TCP/IP协议的实现，能在Super Node(SN)、Proxy Node(PN)的帮助下完成NAT穿透和链路优化，实现极高的节点连通性和更比TCP更好的传输性能。
7. 系统可以有大量不特定的Super Node,Proxy Node的提供者，在激励系统的帮助下，整个网络能保持去中心的组织方式和较高的整体可用性

## 基于DHT的去中心化 Named Object Network
8. BDT网络提供了Named Object的Set/Get能力。大量善意节点组成了稳定的去中心DHT结构，用来保存 Named Object.
9. 在DeviceId的PoW设计帮助下，恶意节点需要污染DHT结构，需要有极大的计算力（超过系统中33%善意节点的算力）。

## 在广域网提供了数据广播能力，包括面向Zone的广播和面向Group的Real Time广播（音视频实时通话）
10. 基于DeviceId中的AreaCode 设计，可以在广域网提供面向特定Zone（同局域网、同城、同省）的boradcast能力。并且这种广播在减少冗余的同时，有更小的延迟特性
11. 提供基于Groupid的中，小范围可控广播,方便网络应用开发
12. 基于P2P 网络传输编码的广播，能在更短的时间里完成大数据块的广播，并减少广播源的带宽小号。提供Real Time Named Data广播

## 实现NDN （Named Data Network)
13. 提供Named Data传输服务(Data Channel)，实现了一个可用的NDN。数据的需求方关注数据准确性而不关注数据来源。
14. Data Channel的设计，为“中间人”加速，数据路由节点的“数据缓存”提供专门的支持。

# 常见缩写
LN : Local Node,通常是发起操作的一方。    
RN : Remote Node,通常是收到请求（类似服务）的一方。    
SN : Super Node 的缩写。    
PN : Proxy Node 的缩写。    
NON : Named Object Network的缩写。    
NDN : Named Data Network的缩写。

# 主要概念解释

## NON(named object network)
NON是基于BDT实现的P2P网络的首要目标，定义了`对象`,特别是`有权对象`.再基于这些对象的语义关系构建`有权对象语义网络`
NON提供的新的结构化信息存储结构，和基于NON的应用开发理论，是系统最重要的理论基础。    
理解NON是理解系统整体设计的基础，请参看专门的文章，来建立对NON的完整了解。

## Groupid (gid):
代表一组DeviceId.通常是BDT里广播操作的"目的"参数。
目前除了SimpleGroup,系统里的各类Group还未完全稳定，需要在实际的应用开发过程中逐步完善。    

## PoW流程
持有DeviceId的设备，应该在空闲时定期进行PoW运算。运算方法是：   
不断变换一个64byte 的 Nonce值，让Hash(Object.Desc + Sign(nonce) + nonce)的值 小于一个指定的难度值.
(给nonce签名的部分可以防止诞生专门的PoW计算服务，这个设计还未完全确定好)。
需要的时候，可以展示这个符合难度规则的nonce值，以让Peerid满足服务提供者设定的基本条件。  
这个设计让系统能区分长期存在的善良节点，让BDT从原理上能解决P2P网络中的`节点信用`问题。    


## Area 区域:
通用用来作为广播操作的目的地参数，和Group不同的是，Zone基本上没有Owner,而是类似“同城、同省的概念”。Area中包含一组Peer.     
Area可以分为逻辑Zone和物理Zone两大类。       
物理区域信息在Desc的区域编码(AreaCode)中，所以DeviceId的物理区域一旦确定就是不可修改的。  
每个DeviceId都只属于一个物理区域。      
逻辑Area通常和由物理网络提供，比如“同一个局域网”，“同一个IPv4网段”。     
每个DeviceId可以属于多个逻辑区域 。      

## Datagram 报文：
通常代表大小为1400字节的逻辑最小传输单元。 LN往RN投递一个Datagram,通常代表着一次低开销的快速通信。
在实际实现中，报文可以通过分片的方法支持更大的大小（8KB?)，同时在底层协议的帮助下，也有一定的可靠性支持。
Datagram可用于一对一通信和一对多通信。

## BDT Package 数据包(package):
BDT协议中定义的逻辑数据包，BDT2.0协议设计文档主要就是定义这些package的详细格式。一个DataPackage的大小不能超过MTU（1460字节)。
BDT Package面向的是BDT的实现，而Datagram是BDT提供的一个可供上层使用网络语义抽象。


## Broadcast 广播：
向一个指定区域或Group广播内容。
从内容的角度有两种主要形式：
一种是Datagram广播，其网络语义与传统的UDP广播类似。通过这种方式广播的数字内容有最大长度限制。     
另一种是Chunk广播，是BDT支持的全新广播语义。如果对延迟的要求高，那么可以用于`音视频多人会议`等直播场景。如果对延迟的要求不高，那么可用于传统的`群内文件分享`场景。
低延迟Chunk广播的具体实现方式将在应用开发中进行验证。会通过内置`网络编码`来提高性能，简化调度算法，降低广播源的带宽成本。      
进行高延迟Chunk广播的逻辑是先用报文广播广播数据头(Data Name)，当接受到数据头广播的节点判断自己需要这个数据的时候自己向广播发起方请求。        
广播的时候如果是基于D-DHT表，实际上是在进行一次点对点遍历发送。相对于网络层提供的原生广播(比如UDP广播）能力，性能难免有所下降，但会有更好的广域网能力。    
进行低延迟chunk广播时，可能需要先组网来保证一定的点对点分片转发可达性。     

## Tunnel 通道：
连接两个节点的无方向、全双工逻辑信道（可以想象成一条虚拟的链路）。两个节点之间建立和保持Tunnel的逻辑属于BDT协议实现的核心策略。每个节点都可以根据自己的实际情况进行选择。   
两个节点之间允许有多条Tunnel.从BDT层的网络语言实现层(Session层）来看，应该更好的利用好已经存在的多条Tunnel.  
当两个Device之间不存在任何可用的Tunnel时，一个Peer想往另一个Peer发送BDT Package,就会触发Tunnel的Connector逻辑。Tunnel的Connector逻辑是BDT协议设计中最复杂的部分。    

Session层实现利用Tunnel来发送BDT Pacakge,底层不提供任何额外的缓存和拥塞控制，逻辑语义基本等价于与UDP的sendto.     
另外，Session层对象的状态（比如Connection的状态）变化可以与tunnel的状态隔离。比如一个Tunnel发生了中断、重连的变化（也许重连后LN\RN的EndPoint发生了变化），但这个变化可以对Connection层是透明的。（这个特性在别的协议里常被称作连接迁移）。   
为了方便Session出了层开发，我们抽象了TunnelContainer.每个Peer对之间只有一个 TunnelContainer，实际开发中我们多使用TunnelContainer的接口来进行Session层的实现。   


# BDT提供的应用层网络语义抽象

## Connection 连接 （Stream）:
与TCP Connection设计一样。提供了LN<->RN的全双工，顺序可靠传输。
在Connection上传输大量数据时，会有拥塞控制机制。BDT允许在不同的Connection上根据其场景使用不同的拥塞控制算法。
一般的，Connection用来传递逻辑控制命令而非Named Data数据。我们相信未来BDT网络中跑在Connection上的流量占比不会超过50%。

## 投递Datagram :
类似UDP的sendto,recvfrom，但能支持更大的Package。


## Boradcast :
类似UDP在局域网的功能，但能在广域网工作。

## SuperNode  SN:
在带NAT 的P2P环境中， 帮助peer与peer建立直接连接的tunnel/channel. Peer通过服务发现机制找到SN，并确定自己是否需要在该SN上注册。一个Peer可以在多个SN上注册，但会消耗额外的带宽成本。

## DHT :
分布式哈希表，是系统实现去中心化的关键基础设施。分布式哈希表为在公网上进行广播提供了基础的支持，并在此基础上提供了一个免费的Global Object Cache.
DHT Ping: 加入DHT的Peer之间用来进行定向的DHT信息交换。
DHT Peer Cache: 使用DHT的机制来保存PeerDesc,发现可用的SN,PN,进而实现BDT网络的“善意去中心化”
DHT Stroage: 把DHT当成一个去中心化的K-V系统，保存任意值（传统DHT实现，可能会被DHT Object-Cache取代）
DHT Object Cache:
DHT是去中心化P2P系统的核心机制，但多年来改进甚少。我们期望在引入经济系统和带PoW的Deviceid后，可以进一步提高DHT机制的性能和可靠性（去中心化能力）、并防止恶意节点对DHT网络的污染。
相关理论已经完成了最基础的核心设计，但DHT的实现细节，还需要在足够规模的实践测试中反复锤炼。
详细的原理描述将有专门的文章介绍。

### Discover 服务发现:
一个peer要在网络中正常工作，或则工作的更好，会依赖其它节点提供的服务。BDT能基于自身的通用机制来发现这些服务（未来可以通过交易市场等更多的方式来发现服务）：   
```
peerlist = discover(peerid,serviceName,gid | area)   // 在一个区域内查找能给peerid提供指定服务的peer列表。
```
发现过程有三种：   
1.中心化调度发现   
2.基于DHT的去中心化查找   
3.基于经济激励的交易市场。  
   
实际使用中的主要问题在于，在不同的peer上执行同样参数的discover,可能会得到不同的列表。这个不一致性可能会严重影响系统的性能，这在去中心化结构下尤其的明显。期望引入基于区块链的经济系统后能有效的解决这个问题。   

## Named Data Network (NDN):
在NON网络中，有定义FileObj,DirObj,DiffObj。这些对象已经是符合NON规范的结构化对象。但在其具体定义中，可以包含或引用非结构化的二进制数据。我们把一个指定长度的数据块称作ChunkData或Chunk.Chunk的objid形式包含其 Hash和Length。FileObj,DirObj,DiffObj,ChunkData 四个核心概念一起构成了。Named Data. 当device通过objid请求NamedData时，由于其拥有对NamedData进行校验的能力，所以可以不关注数据的来源。

### DataChanel (Channel):
Peer与Peer之间建立的用来传输NamedData的通道，与Connection类似。但是为传输NamedData进行了专门的设计。

### DataSession :
在NamedData上建立的一个具体的NamedData的传输会话。传输会话是逻辑意义上的，并无传输方向性。

### ChunkManager:
协议里没有体现，属于实现部分。通常在Device上会运行一个DataMananger来与DataSession的接口进行互相设置。可以全局性（跨进程）的管理设备上的所有NamedData.

## ProxyNode 代理节点(Proxy)：
LN如果无法和RN建立Tunnel或Channel,那么就可以通过P3提供的代理服务来建立Tunnel和Channel.如果P3支持Channel代理，那么当P1,问P2请求的数据P3拥有时，P3可以直接发送数据。
一个ProxyNode还可以支持多种标准Proxy协议。

# 服务提供证明 
基于后付费的服务证明的通用设计

## Step1 买卖双方创建一个可公开的，双方签名的契约。
这是一个买卖双方签名并保存在各自的磁盘上的数字合同，创建后`并不需要保存在链上`。这个数字合同的内容是“买家B 以xxx价格购买了一段时间内卖家S向C提供的服务X（X的详细内容，一般是技术指标）”。    
注意这里有三方，会存在B购买服务给C使用的情况，但很多时候，B和C是同一个用户。       
这个契约不保存着链上的风险在于后付费的情况下，B可能在服务结束后没有足够的余额进行支付。要不要上链取决于S对B的信用判断，如果上联的话，可能S还会需要B在契约内打入`保证金`.上链手续费由B、S协商。      
存储类服务还要求S提供`保证金`，因为S一旦`中途违约`，丢失了保存的数据，那么可能会给B带来很大的损失。    
建立契约的过程和建立闪电网络的支付通道类似，在有保证金的情况下，实际上创造的是一个`共享账户`。这个共享账户在确认一方违约的情况下，可以把账户余额全数转给另一方。    
 
## Step2 契约建立后，S就要开始给C提供服务了。
在提供服务的过程中，S会通过P2P Tunnel不断的问C索要`C签名`的`已服务证明`，这个服务证明是不需要上链的。索要的过程也可以是C定期提供。
如果C不愿意提供“已服务证明”，那么S可以立刻中断服务。 如果C觉得S的服务质量很差，也可以不提供“已服务证明”。
如果C和B是同一个人，B也可以通过直接付一个阶段性费用的方法来表达自己对服务满意。
 
## Step3 付款
如果B对S的服务没有意见，那么在契约时间到达的时候可以按契约约定的（价格*用量）给S付款。否则不付款。
如果S确实提供了高质量的服务，但B迟迟不付款，S可以把`契约`和所有收到的`C签名的已服务证明`提交到链上进行仲裁。设计良好的服务证明机制可以显然的证明S的服务是高质量的，仲裁机构会扣除B的保证金，或降低B的信用，来保障S的利益。

## 请假机制
在契约约定的时间内，S/B/C 因为其它愿意不能主动履约的，可以提出暂时请假。请假条多签名后生效。

## 小结
服务证明机制是通过经济激励调度系统中服务提供者和服务使用者的基础设计，其实现细节依旧需要在实际场景中进行打磨和调整。
### 优点
服务证明的设计的优点是“链下操作”，在买卖双方互信的情况下，进行交易只有支付操作上链，对链的负载低。
服务证明的设计并不要求100%的可靠，C愿不愿出示服务证明可以有自己的独立算法，并不要求做100%可靠的检测。
另一个优势是提供了一定的“协商空间”，如果B、C、S对服务的质量问题能通过其他渠道达成一致，也不用设计链上机制处理

### 缺点
服务证明很多时候是基于统计的，不是100%可靠的，这里可能会给系统带来一些整体性的风险。
存在C和S一起串通，坑B的可能性。这需要B慎重的选择C。这需要B设计合理的机制来慎重的选择C，或则给每个C一个积分上限，防止单个C消耗太多的资源。
后付费证明通常允许服务的使用者至少免费试用一次服务，理论上基于PoW的Peerid生成不会让这个机制被滥用。我们还会考虑建立`对象信用`机制（基于存证理论）来进一步减少`白嫖`。

# BDT 系统里的其它标准服务

## DataTracker:
用来查询哪些Peer拥有指定NamedData的服务（哪些Peer拥有NamedData,Peer拥有哪些NamedData）。能提供比纯DHT查找更高的性能。
同时还能查询NamedData与NamedData之间的关系。比如NamedData中包含哪些SubNamedData,SubNameData是否属于某个NamedData,两个不同命名方式的NamedData是否相等等信息。

## PeerStatistics(Histroy):
在设备上保存的别的Peer的状态。里面除了由通过PeerInfo推断出来的状态，还有通过实际通信得到的状态（比如最后一次收到通信的EP和时间）

## PeerAbility (pAbility)
节点的能力支持情况。通常用来由Peer自己提供。通过一个u64数的每一个bit来描述一些常见的能力
```
{
A0_haveNAT
A1_supportUDP
A2_supportTCP
A3_A4_cryptoLevel
A5_SNService
A6_TunnelProxy
A7_ChannelProxy
A8_supportDHT
... 待完善
}
```